from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.agentic_workflow_success_scorer import AgenticWorkflowSuccessScorer
    from ..models.bleu_scorer import BleuScorer
    from ..models.chunk_attribution_utilization_scorer import ChunkAttributionUtilizationScorer
    from ..models.completeness_scorer import CompletenessScorer
    from ..models.context_adherence_scorer import ContextAdherenceScorer
    from ..models.context_relevance_scorer import ContextRelevanceScorer
    from ..models.correctness_scorer import CorrectnessScorer
    from ..models.generated_scorer import GeneratedScorer
    from ..models.ground_truth_adherence_scorer import GroundTruthAdherenceScorer
    from ..models.input_pii_scorer import InputPIIScorer
    from ..models.input_sexist_scorer import InputSexistScorer
    from ..models.input_tone_scorer import InputToneScorer
    from ..models.input_toxicity_scorer import InputToxicityScorer
    from ..models.instruction_adherence_scorer import InstructionAdherenceScorer
    from ..models.output_pii_scorer import OutputPIIScorer
    from ..models.output_sexist_scorer import OutputSexistScorer
    from ..models.output_tone_scorer import OutputToneScorer
    from ..models.output_toxicity_scorer import OutputToxicityScorer
    from ..models.prompt_injection_scorer import PromptInjectionScorer
    from ..models.prompt_perplexity_scorer import PromptPerplexityScorer
    from ..models.registered_scorer import RegisteredScorer
    from ..models.rouge_scorer import RougeScorer
    from ..models.tool_error_rate_scorer import ToolErrorRateScorer
    from ..models.tool_selection_quality_scorer import ToolSelectionQualityScorer
    from ..models.uncertainty_scorer import UncertaintyScorer


T = TypeVar("T", bound="ScorersConfig")


@_attrs_define
class ScorersConfig:
    """
    Attributes:
        generated_scorers (Union[Unset, list['GeneratedScorer']]): List of user generated scorers to enable.
        registered_scorers (Union[Unset, list['RegisteredScorer']]): List of user registered scorers to enable.
        scorers (Union[Unset, list[Union['AgenticWorkflowSuccessScorer', 'BleuScorer',
            'ChunkAttributionUtilizationScorer', 'CompletenessScorer', 'ContextAdherenceScorer', 'ContextRelevanceScorer',
            'CorrectnessScorer', 'GroundTruthAdherenceScorer', 'InputPIIScorer', 'InputSexistScorer', 'InputToneScorer',
            'InputToxicityScorer', 'InstructionAdherenceScorer', 'OutputPIIScorer', 'OutputSexistScorer',
            'OutputToneScorer', 'OutputToxicityScorer', 'PromptInjectionScorer', 'PromptPerplexityScorer', 'RougeScorer',
            'ToolErrorRateScorer', 'ToolSelectionQualityScorer', 'UncertaintyScorer']]]): List of Galileo scorers to enable.
    """

    generated_scorers: Union[Unset, list["GeneratedScorer"]] = UNSET
    registered_scorers: Union[Unset, list["RegisteredScorer"]] = UNSET
    scorers: Union[
        Unset,
        list[
            Union[
                "AgenticWorkflowSuccessScorer",
                "BleuScorer",
                "ChunkAttributionUtilizationScorer",
                "CompletenessScorer",
                "ContextAdherenceScorer",
                "ContextRelevanceScorer",
                "CorrectnessScorer",
                "GroundTruthAdherenceScorer",
                "InputPIIScorer",
                "InputSexistScorer",
                "InputToneScorer",
                "InputToxicityScorer",
                "InstructionAdherenceScorer",
                "OutputPIIScorer",
                "OutputSexistScorer",
                "OutputToneScorer",
                "OutputToxicityScorer",
                "PromptInjectionScorer",
                "PromptPerplexityScorer",
                "RougeScorer",
                "ToolErrorRateScorer",
                "ToolSelectionQualityScorer",
                "UncertaintyScorer",
            ]
        ],
    ] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.agentic_workflow_success_scorer import AgenticWorkflowSuccessScorer
        from ..models.bleu_scorer import BleuScorer
        from ..models.chunk_attribution_utilization_scorer import ChunkAttributionUtilizationScorer
        from ..models.completeness_scorer import CompletenessScorer
        from ..models.context_adherence_scorer import ContextAdherenceScorer
        from ..models.context_relevance_scorer import ContextRelevanceScorer
        from ..models.correctness_scorer import CorrectnessScorer
        from ..models.ground_truth_adherence_scorer import GroundTruthAdherenceScorer
        from ..models.input_pii_scorer import InputPIIScorer
        from ..models.input_sexist_scorer import InputSexistScorer
        from ..models.input_tone_scorer import InputToneScorer
        from ..models.input_toxicity_scorer import InputToxicityScorer
        from ..models.instruction_adherence_scorer import InstructionAdherenceScorer
        from ..models.output_pii_scorer import OutputPIIScorer
        from ..models.output_sexist_scorer import OutputSexistScorer
        from ..models.output_tone_scorer import OutputToneScorer
        from ..models.output_toxicity_scorer import OutputToxicityScorer
        from ..models.prompt_injection_scorer import PromptInjectionScorer
        from ..models.prompt_perplexity_scorer import PromptPerplexityScorer
        from ..models.rouge_scorer import RougeScorer
        from ..models.tool_error_rate_scorer import ToolErrorRateScorer
        from ..models.tool_selection_quality_scorer import ToolSelectionQualityScorer

        generated_scorers: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.generated_scorers, Unset):
            generated_scorers = []
            for generated_scorers_item_data in self.generated_scorers:
                generated_scorers_item = generated_scorers_item_data.to_dict()
                generated_scorers.append(generated_scorers_item)

        registered_scorers: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.registered_scorers, Unset):
            registered_scorers = []
            for registered_scorers_item_data in self.registered_scorers:
                registered_scorers_item = registered_scorers_item_data.to_dict()
                registered_scorers.append(registered_scorers_item)

        scorers: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.scorers, Unset):
            scorers = []
            for scorers_item_data in self.scorers:
                scorers_item: dict[str, Any]
                if isinstance(scorers_item_data, AgenticWorkflowSuccessScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, BleuScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, ChunkAttributionUtilizationScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, CompletenessScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, ContextAdherenceScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, ContextRelevanceScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, CorrectnessScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, GroundTruthAdherenceScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, InputPIIScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, InputSexistScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, InputToneScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, InputToxicityScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, InstructionAdherenceScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, OutputPIIScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, OutputSexistScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, OutputToneScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, OutputToxicityScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, PromptInjectionScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, PromptPerplexityScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, RougeScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, ToolErrorRateScorer):
                    scorers_item = scorers_item_data.to_dict()
                elif isinstance(scorers_item_data, ToolSelectionQualityScorer):
                    scorers_item = scorers_item_data.to_dict()
                else:
                    scorers_item = scorers_item_data.to_dict()

                scorers.append(scorers_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if generated_scorers is not UNSET:
            field_dict["generated_scorers"] = generated_scorers
        if registered_scorers is not UNSET:
            field_dict["registered_scorers"] = registered_scorers
        if scorers is not UNSET:
            field_dict["scorers"] = scorers

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.agentic_workflow_success_scorer import AgenticWorkflowSuccessScorer
        from ..models.bleu_scorer import BleuScorer
        from ..models.chunk_attribution_utilization_scorer import ChunkAttributionUtilizationScorer
        from ..models.completeness_scorer import CompletenessScorer
        from ..models.context_adherence_scorer import ContextAdherenceScorer
        from ..models.context_relevance_scorer import ContextRelevanceScorer
        from ..models.correctness_scorer import CorrectnessScorer
        from ..models.generated_scorer import GeneratedScorer
        from ..models.ground_truth_adherence_scorer import GroundTruthAdherenceScorer
        from ..models.input_pii_scorer import InputPIIScorer
        from ..models.input_sexist_scorer import InputSexistScorer
        from ..models.input_tone_scorer import InputToneScorer
        from ..models.input_toxicity_scorer import InputToxicityScorer
        from ..models.instruction_adherence_scorer import InstructionAdherenceScorer
        from ..models.output_pii_scorer import OutputPIIScorer
        from ..models.output_sexist_scorer import OutputSexistScorer
        from ..models.output_tone_scorer import OutputToneScorer
        from ..models.output_toxicity_scorer import OutputToxicityScorer
        from ..models.prompt_injection_scorer import PromptInjectionScorer
        from ..models.prompt_perplexity_scorer import PromptPerplexityScorer
        from ..models.registered_scorer import RegisteredScorer
        from ..models.rouge_scorer import RougeScorer
        from ..models.tool_error_rate_scorer import ToolErrorRateScorer
        from ..models.tool_selection_quality_scorer import ToolSelectionQualityScorer
        from ..models.uncertainty_scorer import UncertaintyScorer

        d = src_dict.copy()
        generated_scorers = []
        _generated_scorers = d.pop("generated_scorers", UNSET)
        for generated_scorers_item_data in _generated_scorers or []:
            generated_scorers_item = GeneratedScorer.from_dict(generated_scorers_item_data)

            generated_scorers.append(generated_scorers_item)

        registered_scorers = []
        _registered_scorers = d.pop("registered_scorers", UNSET)
        for registered_scorers_item_data in _registered_scorers or []:
            registered_scorers_item = RegisteredScorer.from_dict(registered_scorers_item_data)

            registered_scorers.append(registered_scorers_item)

        scorers = []
        _scorers = d.pop("scorers", UNSET)
        for scorers_item_data in _scorers or []:

            def _parse_scorers_item(
                data: object,
            ) -> Union[
                "AgenticWorkflowSuccessScorer",
                "BleuScorer",
                "ChunkAttributionUtilizationScorer",
                "CompletenessScorer",
                "ContextAdherenceScorer",
                "ContextRelevanceScorer",
                "CorrectnessScorer",
                "GroundTruthAdherenceScorer",
                "InputPIIScorer",
                "InputSexistScorer",
                "InputToneScorer",
                "InputToxicityScorer",
                "InstructionAdherenceScorer",
                "OutputPIIScorer",
                "OutputSexistScorer",
                "OutputToneScorer",
                "OutputToxicityScorer",
                "PromptInjectionScorer",
                "PromptPerplexityScorer",
                "RougeScorer",
                "ToolErrorRateScorer",
                "ToolSelectionQualityScorer",
                "UncertaintyScorer",
            ]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_0 = AgenticWorkflowSuccessScorer.from_dict(data)

                    return scorers_item_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_1 = BleuScorer.from_dict(data)

                    return scorers_item_type_1
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_2 = ChunkAttributionUtilizationScorer.from_dict(data)

                    return scorers_item_type_2
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_3 = CompletenessScorer.from_dict(data)

                    return scorers_item_type_3
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_4 = ContextAdherenceScorer.from_dict(data)

                    return scorers_item_type_4
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_5 = ContextRelevanceScorer.from_dict(data)

                    return scorers_item_type_5
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_6 = CorrectnessScorer.from_dict(data)

                    return scorers_item_type_6
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_7 = GroundTruthAdherenceScorer.from_dict(data)

                    return scorers_item_type_7
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_8 = InputPIIScorer.from_dict(data)

                    return scorers_item_type_8
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_9 = InputSexistScorer.from_dict(data)

                    return scorers_item_type_9
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_10 = InputToneScorer.from_dict(data)

                    return scorers_item_type_10
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_11 = InputToxicityScorer.from_dict(data)

                    return scorers_item_type_11
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_12 = InstructionAdherenceScorer.from_dict(data)

                    return scorers_item_type_12
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_13 = OutputPIIScorer.from_dict(data)

                    return scorers_item_type_13
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_14 = OutputSexistScorer.from_dict(data)

                    return scorers_item_type_14
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_15 = OutputToneScorer.from_dict(data)

                    return scorers_item_type_15
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_16 = OutputToxicityScorer.from_dict(data)

                    return scorers_item_type_16
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_17 = PromptInjectionScorer.from_dict(data)

                    return scorers_item_type_17
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_18 = PromptPerplexityScorer.from_dict(data)

                    return scorers_item_type_18
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_19 = RougeScorer.from_dict(data)

                    return scorers_item_type_19
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_20 = ToolErrorRateScorer.from_dict(data)

                    return scorers_item_type_20
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    scorers_item_type_21 = ToolSelectionQualityScorer.from_dict(data)

                    return scorers_item_type_21
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                scorers_item_type_22 = UncertaintyScorer.from_dict(data)

                return scorers_item_type_22

            scorers_item = _parse_scorers_item(scorers_item_data)

            scorers.append(scorers_item)

        scorers_config = cls(
            generated_scorers=generated_scorers, registered_scorers=registered_scorers, scorers=scorers
        )

        scorers_config.additional_properties = d
        return scorers_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.alerts_configuration import AlertsConfiguration
    from ..models.composite_project_settings_metric_weights_configuration_type_0 import (
        CompositeProjectSettingsMetricWeightsConfigurationType0,
    )
    from ..models.customized_scorer import CustomizedScorer
    from ..models.generated_scorer_config import GeneratedScorerConfig
    from ..models.registered_scorer import RegisteredScorer
    from ..models.scorers_config import ScorersConfig
    from ..models.scorers_configuration import ScorersConfiguration


T = TypeVar("T", bound="CompositeProjectSettings")


@_attrs_define
class CompositeProjectSettings:
    """
    Attributes:
        alerts_configuration (Union['AlertsConfiguration', None, Unset]):
        customized_scorers_configuration (Union[None, Unset, list['CustomizedScorer']]):
        generated_scorers_configuration (Union[None, Unset, list['GeneratedScorerConfig']]):
        metric_weights_configuration (Union['CompositeProjectSettingsMetricWeightsConfigurationType0', None, Unset]):
        registered_scorers_configuration (Union[None, Unset, list['RegisteredScorer']]):
        scorers_config (Union['ScorersConfig', None, Unset]):
        scorers_configuration (Union['ScorersConfiguration', None, Unset]):
    """

    alerts_configuration: Union["AlertsConfiguration", None, Unset] = UNSET
    customized_scorers_configuration: Union[None, Unset, list["CustomizedScorer"]] = UNSET
    generated_scorers_configuration: Union[None, Unset, list["GeneratedScorerConfig"]] = UNSET
    metric_weights_configuration: Union["CompositeProjectSettingsMetricWeightsConfigurationType0", None, Unset] = UNSET
    registered_scorers_configuration: Union[None, Unset, list["RegisteredScorer"]] = UNSET
    scorers_config: Union["ScorersConfig", None, Unset] = UNSET
    scorers_configuration: Union["ScorersConfiguration", None, Unset] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.alerts_configuration import AlertsConfiguration
        from ..models.composite_project_settings_metric_weights_configuration_type_0 import (
            CompositeProjectSettingsMetricWeightsConfigurationType0,
        )
        from ..models.scorers_config import ScorersConfig
        from ..models.scorers_configuration import ScorersConfiguration

        alerts_configuration: Union[None, Unset, dict[str, Any]]
        if isinstance(self.alerts_configuration, Unset):
            alerts_configuration = UNSET
        elif isinstance(self.alerts_configuration, AlertsConfiguration):
            alerts_configuration = self.alerts_configuration.to_dict()
        else:
            alerts_configuration = self.alerts_configuration

        customized_scorers_configuration: Union[None, Unset, list[dict[str, Any]]]
        if isinstance(self.customized_scorers_configuration, Unset):
            customized_scorers_configuration = UNSET
        elif isinstance(self.customized_scorers_configuration, list):
            customized_scorers_configuration = []
            for customized_scorers_configuration_type_0_item_data in self.customized_scorers_configuration:
                customized_scorers_configuration_type_0_item = (
                    customized_scorers_configuration_type_0_item_data.to_dict()
                )
                customized_scorers_configuration.append(customized_scorers_configuration_type_0_item)

        else:
            customized_scorers_configuration = self.customized_scorers_configuration

        generated_scorers_configuration: Union[None, Unset, list[dict[str, Any]]]
        if isinstance(self.generated_scorers_configuration, Unset):
            generated_scorers_configuration = UNSET
        elif isinstance(self.generated_scorers_configuration, list):
            generated_scorers_configuration = []
            for generated_scorers_configuration_type_0_item_data in self.generated_scorers_configuration:
                generated_scorers_configuration_type_0_item = generated_scorers_configuration_type_0_item_data.to_dict()
                generated_scorers_configuration.append(generated_scorers_configuration_type_0_item)

        else:
            generated_scorers_configuration = self.generated_scorers_configuration

        metric_weights_configuration: Union[None, Unset, dict[str, Any]]
        if isinstance(self.metric_weights_configuration, Unset):
            metric_weights_configuration = UNSET
        elif isinstance(self.metric_weights_configuration, CompositeProjectSettingsMetricWeightsConfigurationType0):
            metric_weights_configuration = self.metric_weights_configuration.to_dict()
        else:
            metric_weights_configuration = self.metric_weights_configuration

        registered_scorers_configuration: Union[None, Unset, list[dict[str, Any]]]
        if isinstance(self.registered_scorers_configuration, Unset):
            registered_scorers_configuration = UNSET
        elif isinstance(self.registered_scorers_configuration, list):
            registered_scorers_configuration = []
            for registered_scorers_configuration_type_0_item_data in self.registered_scorers_configuration:
                registered_scorers_configuration_type_0_item = (
                    registered_scorers_configuration_type_0_item_data.to_dict()
                )
                registered_scorers_configuration.append(registered_scorers_configuration_type_0_item)

        else:
            registered_scorers_configuration = self.registered_scorers_configuration

        scorers_config: Union[None, Unset, dict[str, Any]]
        if isinstance(self.scorers_config, Unset):
            scorers_config = UNSET
        elif isinstance(self.scorers_config, ScorersConfig):
            scorers_config = self.scorers_config.to_dict()
        else:
            scorers_config = self.scorers_config

        scorers_configuration: Union[None, Unset, dict[str, Any]]
        if isinstance(self.scorers_configuration, Unset):
            scorers_configuration = UNSET
        elif isinstance(self.scorers_configuration, ScorersConfiguration):
            scorers_configuration = self.scorers_configuration.to_dict()
        else:
            scorers_configuration = self.scorers_configuration

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if alerts_configuration is not UNSET:
            field_dict["alerts_configuration"] = alerts_configuration
        if customized_scorers_configuration is not UNSET:
            field_dict["customized_scorers_configuration"] = customized_scorers_configuration
        if generated_scorers_configuration is not UNSET:
            field_dict["generated_scorers_configuration"] = generated_scorers_configuration
        if metric_weights_configuration is not UNSET:
            field_dict["metric_weights_configuration"] = metric_weights_configuration
        if registered_scorers_configuration is not UNSET:
            field_dict["registered_scorers_configuration"] = registered_scorers_configuration
        if scorers_config is not UNSET:
            field_dict["scorers_config"] = scorers_config
        if scorers_configuration is not UNSET:
            field_dict["scorers_configuration"] = scorers_configuration

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.alerts_configuration import AlertsConfiguration
        from ..models.composite_project_settings_metric_weights_configuration_type_0 import (
            CompositeProjectSettingsMetricWeightsConfigurationType0,
        )
        from ..models.customized_scorer import CustomizedScorer
        from ..models.generated_scorer_config import GeneratedScorerConfig
        from ..models.registered_scorer import RegisteredScorer
        from ..models.scorers_config import ScorersConfig
        from ..models.scorers_configuration import ScorersConfiguration

        d = src_dict.copy()

        def _parse_alerts_configuration(data: object) -> Union["AlertsConfiguration", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                alerts_configuration_type_0 = AlertsConfiguration.from_dict(data)

                return alerts_configuration_type_0
            except:  # noqa: E722
                pass
            return cast(Union["AlertsConfiguration", None, Unset], data)

        alerts_configuration = _parse_alerts_configuration(d.pop("alerts_configuration", UNSET))

        def _parse_customized_scorers_configuration(data: object) -> Union[None, Unset, list["CustomizedScorer"]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                customized_scorers_configuration_type_0 = []
                _customized_scorers_configuration_type_0 = data
                for customized_scorers_configuration_type_0_item_data in _customized_scorers_configuration_type_0:
                    customized_scorers_configuration_type_0_item = CustomizedScorer.from_dict(
                        customized_scorers_configuration_type_0_item_data
                    )

                    customized_scorers_configuration_type_0.append(customized_scorers_configuration_type_0_item)

                return customized_scorers_configuration_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list["CustomizedScorer"]], data)

        customized_scorers_configuration = _parse_customized_scorers_configuration(
            d.pop("customized_scorers_configuration", UNSET)
        )

        def _parse_generated_scorers_configuration(data: object) -> Union[None, Unset, list["GeneratedScorerConfig"]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                generated_scorers_configuration_type_0 = []
                _generated_scorers_configuration_type_0 = data
                for generated_scorers_configuration_type_0_item_data in _generated_scorers_configuration_type_0:
                    generated_scorers_configuration_type_0_item = GeneratedScorerConfig.from_dict(
                        generated_scorers_configuration_type_0_item_data
                    )

                    generated_scorers_configuration_type_0.append(generated_scorers_configuration_type_0_item)

                return generated_scorers_configuration_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list["GeneratedScorerConfig"]], data)

        generated_scorers_configuration = _parse_generated_scorers_configuration(
            d.pop("generated_scorers_configuration", UNSET)
        )

        def _parse_metric_weights_configuration(
            data: object,
        ) -> Union["CompositeProjectSettingsMetricWeightsConfigurationType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metric_weights_configuration_type_0 = CompositeProjectSettingsMetricWeightsConfigurationType0.from_dict(
                    data
                )

                return metric_weights_configuration_type_0
            except:  # noqa: E722
                pass
            return cast(Union["CompositeProjectSettingsMetricWeightsConfigurationType0", None, Unset], data)

        metric_weights_configuration = _parse_metric_weights_configuration(d.pop("metric_weights_configuration", UNSET))

        def _parse_registered_scorers_configuration(data: object) -> Union[None, Unset, list["RegisteredScorer"]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                registered_scorers_configuration_type_0 = []
                _registered_scorers_configuration_type_0 = data
                for registered_scorers_configuration_type_0_item_data in _registered_scorers_configuration_type_0:
                    registered_scorers_configuration_type_0_item = RegisteredScorer.from_dict(
                        registered_scorers_configuration_type_0_item_data
                    )

                    registered_scorers_configuration_type_0.append(registered_scorers_configuration_type_0_item)

                return registered_scorers_configuration_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list["RegisteredScorer"]], data)

        registered_scorers_configuration = _parse_registered_scorers_configuration(
            d.pop("registered_scorers_configuration", UNSET)
        )

        def _parse_scorers_config(data: object) -> Union["ScorersConfig", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                scorers_config_type_0 = ScorersConfig.from_dict(data)

                return scorers_config_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ScorersConfig", None, Unset], data)

        scorers_config = _parse_scorers_config(d.pop("scorers_config", UNSET))

        def _parse_scorers_configuration(data: object) -> Union["ScorersConfiguration", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                scorers_configuration_type_0 = ScorersConfiguration.from_dict(data)

                return scorers_configuration_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ScorersConfiguration", None, Unset], data)

        scorers_configuration = _parse_scorers_configuration(d.pop("scorers_configuration", UNSET))

        composite_project_settings = cls(
            alerts_configuration=alerts_configuration,
            customized_scorers_configuration=customized_scorers_configuration,
            generated_scorers_configuration=generated_scorers_configuration,
            metric_weights_configuration=metric_weights_configuration,
            registered_scorers_configuration=registered_scorers_configuration,
            scorers_config=scorers_config,
            scorers_configuration=scorers_configuration,
        )

        composite_project_settings.additional_properties = d
        return composite_project_settings

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.data_row import DataRow
    from ..models.run_results_top_erroneous_words_type_0 import RunResultsTopErroneousWordsType0


T = TypeVar("T", bound="RunResults")


@_attrs_define
class RunResults:
    """
    Attributes:
        has_next_page (bool):
        labels (list[str]):
        sample_count (int):
        sample_easy_percentage (float):
        sample_hard_percentage (float):
        sample_misclassified_percentage (float):
        split_total_sample_count (int):
        data_metrics (Union[Unset, list['DataRow']]):
        easy_samples_threshold (Union[None, Unset, float]):
        hard_samples_threshold (Union[None, Unset, float]):
        span_count (Union[None, Unset, int]):
        split_total_span_count (Union[None, Unset, int]):
        top_erroneous_words (Union['RunResultsTopErroneousWordsType0', None, Unset]):
    """

    has_next_page: bool
    labels: list[str]
    sample_count: int
    sample_easy_percentage: float
    sample_hard_percentage: float
    sample_misclassified_percentage: float
    split_total_sample_count: int
    data_metrics: Union[Unset, list["DataRow"]] = UNSET
    easy_samples_threshold: Union[None, Unset, float] = UNSET
    hard_samples_threshold: Union[None, Unset, float] = UNSET
    span_count: Union[None, Unset, int] = UNSET
    split_total_span_count: Union[None, Unset, int] = UNSET
    top_erroneous_words: Union["RunResultsTopErroneousWordsType0", None, Unset] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.run_results_top_erroneous_words_type_0 import RunResultsTopErroneousWordsType0

        has_next_page = self.has_next_page

        labels = self.labels

        sample_count = self.sample_count

        sample_easy_percentage = self.sample_easy_percentage

        sample_hard_percentage = self.sample_hard_percentage

        sample_misclassified_percentage = self.sample_misclassified_percentage

        split_total_sample_count = self.split_total_sample_count

        data_metrics: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.data_metrics, Unset):
            data_metrics = []
            for data_metrics_item_data in self.data_metrics:
                data_metrics_item = data_metrics_item_data.to_dict()
                data_metrics.append(data_metrics_item)

        easy_samples_threshold: Union[None, Unset, float]
        if isinstance(self.easy_samples_threshold, Unset):
            easy_samples_threshold = UNSET
        else:
            easy_samples_threshold = self.easy_samples_threshold

        hard_samples_threshold: Union[None, Unset, float]
        if isinstance(self.hard_samples_threshold, Unset):
            hard_samples_threshold = UNSET
        else:
            hard_samples_threshold = self.hard_samples_threshold

        span_count: Union[None, Unset, int]
        if isinstance(self.span_count, Unset):
            span_count = UNSET
        else:
            span_count = self.span_count

        split_total_span_count: Union[None, Unset, int]
        if isinstance(self.split_total_span_count, Unset):
            split_total_span_count = UNSET
        else:
            split_total_span_count = self.split_total_span_count

        top_erroneous_words: Union[None, Unset, dict[str, Any]]
        if isinstance(self.top_erroneous_words, Unset):
            top_erroneous_words = UNSET
        elif isinstance(self.top_erroneous_words, RunResultsTopErroneousWordsType0):
            top_erroneous_words = self.top_erroneous_words.to_dict()
        else:
            top_erroneous_words = self.top_erroneous_words

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "has_next_page": has_next_page,
                "labels": labels,
                "sample_count": sample_count,
                "sample_easy_percentage": sample_easy_percentage,
                "sample_hard_percentage": sample_hard_percentage,
                "sample_misclassified_percentage": sample_misclassified_percentage,
                "split_total_sample_count": split_total_sample_count,
            }
        )
        if data_metrics is not UNSET:
            field_dict["data_metrics"] = data_metrics
        if easy_samples_threshold is not UNSET:
            field_dict["easy_samples_threshold"] = easy_samples_threshold
        if hard_samples_threshold is not UNSET:
            field_dict["hard_samples_threshold"] = hard_samples_threshold
        if span_count is not UNSET:
            field_dict["span_count"] = span_count
        if split_total_span_count is not UNSET:
            field_dict["split_total_span_count"] = split_total_span_count
        if top_erroneous_words is not UNSET:
            field_dict["top_erroneous_words"] = top_erroneous_words

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.data_row import DataRow
        from ..models.run_results_top_erroneous_words_type_0 import RunResultsTopErroneousWordsType0

        d = src_dict.copy()
        has_next_page = d.pop("has_next_page")

        labels = cast(list[str], d.pop("labels"))

        sample_count = d.pop("sample_count")

        sample_easy_percentage = d.pop("sample_easy_percentage")

        sample_hard_percentage = d.pop("sample_hard_percentage")

        sample_misclassified_percentage = d.pop("sample_misclassified_percentage")

        split_total_sample_count = d.pop("split_total_sample_count")

        data_metrics = []
        _data_metrics = d.pop("data_metrics", UNSET)
        for data_metrics_item_data in _data_metrics or []:
            data_metrics_item = DataRow.from_dict(data_metrics_item_data)

            data_metrics.append(data_metrics_item)

        def _parse_easy_samples_threshold(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        easy_samples_threshold = _parse_easy_samples_threshold(d.pop("easy_samples_threshold", UNSET))

        def _parse_hard_samples_threshold(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        hard_samples_threshold = _parse_hard_samples_threshold(d.pop("hard_samples_threshold", UNSET))

        def _parse_span_count(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        span_count = _parse_span_count(d.pop("span_count", UNSET))

        def _parse_split_total_span_count(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        split_total_span_count = _parse_split_total_span_count(d.pop("split_total_span_count", UNSET))

        def _parse_top_erroneous_words(data: object) -> Union["RunResultsTopErroneousWordsType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                top_erroneous_words_type_0 = RunResultsTopErroneousWordsType0.from_dict(data)

                return top_erroneous_words_type_0
            except:  # noqa: E722
                pass
            return cast(Union["RunResultsTopErroneousWordsType0", None, Unset], data)

        top_erroneous_words = _parse_top_erroneous_words(d.pop("top_erroneous_words", UNSET))

        run_results = cls(
            has_next_page=has_next_page,
            labels=labels,
            sample_count=sample_count,
            sample_easy_percentage=sample_easy_percentage,
            sample_hard_percentage=sample_hard_percentage,
            sample_misclassified_percentage=sample_misclassified_percentage,
            split_total_sample_count=split_total_sample_count,
            data_metrics=data_metrics,
            easy_samples_threshold=easy_samples_threshold,
            hard_samples_threshold=hard_samples_threshold,
            span_count=span_count,
            split_total_span_count=split_total_span_count,
            top_erroneous_words=top_erroneous_words,
        )

        run_results.additional_properties = d
        return run_results

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class GeneratedScorerTaskResultType(str, Enum):
    SCORER_AUTO_GEN = "scorer_auto_gen"
    SCORER_VALIDATION = "scorer_validation"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class GenericAction(str, Enum):
    GENERIC_CREATE = "generic_create"
    GENERIC_DELETE = "generic_delete"
    GENERIC_READ = "generic_read"
    GENERIC_UPDATE = "generic_update"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.llm_export_format import LLMExportFormat
from ...types import UNSET, Response, Unset


def _get_kwargs(
    project_id: str,
    run_id: str,
    *,
    export_format: Union[Unset, LLMExportFormat] = UNSET,
    include_chains: Union[Unset, bool] = True,
    columns: Union[Unset, list[str]] = UNSET,
    inputs: Union[Unset, bool] = True,
    prompts: Union[Unset, bool] = True,
    responses: Union[Unset, bool] = True,
    metrics: Union[Unset, bool] = True,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    json_export_format: Union[Unset, str] = UNSET
    if not isinstance(export_format, Unset):
        json_export_format = export_format.value

    params["export_format"] = json_export_format

    params["include_chains"] = include_chains

    json_columns: Union[Unset, list[str]] = UNSET
    if not isinstance(columns, Unset):
        json_columns = columns

    params["columns"] = json_columns

    params["inputs"] = inputs

    params["prompts"] = prompts

    params["responses"] = responses

    params["metrics"] = metrics

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": f"/projects/{project_id}/runs/{run_id}/prompts/export_prompt_dataset",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, HTTPValidationError]]:
    if response.status_code == 200:
        response_200 = cast(Any, None)
        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, HTTPValidationError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    project_id: str,
    run_id: str,
    *,
    client: AuthenticatedClient,
    export_format: Union[Unset, LLMExportFormat] = UNSET,
    include_chains: Union[Unset, bool] = True,
    columns: Union[Unset, list[str]] = UNSET,
    inputs: Union[Unset, bool] = True,
    prompts: Union[Unset, bool] = True,
    responses: Union[Unset, bool] = True,
    metrics: Union[Unset, bool] = True,
) -> Response[Union[Any, HTTPValidationError]]:
    """Export Prompt Dataset

    Args:
        project_id (str):
        run_id (str):
        export_format (Union[Unset, LLMExportFormat]):
        include_chains (Union[Unset, bool]):  Default: True.
        columns (Union[Unset, list[str]]):
        inputs (Union[Unset, bool]):  Default: True.
        prompts (Union[Unset, bool]):  Default: True.
        responses (Union[Unset, bool]):  Default: True.
        metrics (Union[Unset, bool]):  Default: True.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, HTTPValidationError]]
    """

    kwargs = _get_kwargs(
        project_id=project_id,
        run_id=run_id,
        export_format=export_format,
        include_chains=include_chains,
        columns=columns,
        inputs=inputs,
        prompts=prompts,
        responses=responses,
        metrics=metrics,
    )

    response = client.get_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


def sync(
    project_id: str,
    run_id: str,
    *,
    client: AuthenticatedClient,
    export_format: Union[Unset, LLMExportFormat] = UNSET,
    include_chains: Union[Unset, bool] = True,
    columns: Union[Unset, list[str]] = UNSET,
    inputs: Union[Unset, bool] = True,
    prompts: Union[Unset, bool] = True,
    responses: Union[Unset, bool] = True,
    metrics: Union[Unset, bool] = True,
) -> Optional[Union[Any, HTTPValidationError]]:
    """Export Prompt Dataset

    Args:
        project_id (str):
        run_id (str):
        export_format (Union[Unset, LLMExportFormat]):
        include_chains (Union[Unset, bool]):  Default: True.
        columns (Union[Unset, list[str]]):
        inputs (Union[Unset, bool]):  Default: True.
        prompts (Union[Unset, bool]):  Default: True.
        responses (Union[Unset, bool]):  Default: True.
        metrics (Union[Unset, bool]):  Default: True.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, HTTPValidationError]
    """

    return sync_detailed(
        project_id=project_id,
        run_id=run_id,
        client=client,
        export_format=export_format,
        include_chains=include_chains,
        columns=columns,
        inputs=inputs,
        prompts=prompts,
        responses=responses,
        metrics=metrics,
    ).parsed


async def asyncio_detailed(
    project_id: str,
    run_id: str,
    *,
    client: AuthenticatedClient,
    export_format: Union[Unset, LLMExportFormat] = UNSET,
    include_chains: Union[Unset, bool] = True,
    columns: Union[Unset, list[str]] = UNSET,
    inputs: Union[Unset, bool] = True,
    prompts: Union[Unset, bool] = True,
    responses: Union[Unset, bool] = True,
    metrics: Union[Unset, bool] = True,
) -> Response[Union[Any, HTTPValidationError]]:
    """Export Prompt Dataset

    Args:
        project_id (str):
        run_id (str):
        export_format (Union[Unset, LLMExportFormat]):
        include_chains (Union[Unset, bool]):  Default: True.
        columns (Union[Unset, list[str]]):
        inputs (Union[Unset, bool]):  Default: True.
        prompts (Union[Unset, bool]):  Default: True.
        responses (Union[Unset, bool]):  Default: True.
        metrics (Union[Unset, bool]):  Default: True.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, HTTPValidationError]]
    """

    kwargs = _get_kwargs(
        project_id=project_id,
        run_id=run_id,
        export_format=export_format,
        include_chains=include_chains,
        columns=columns,
        inputs=inputs,
        prompts=prompts,
        responses=responses,
        metrics=metrics,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    project_id: str,
    run_id: str,
    *,
    client: AuthenticatedClient,
    export_format: Union[Unset, LLMExportFormat] = UNSET,
    include_chains: Union[Unset, bool] = True,
    columns: Union[Unset, list[str]] = UNSET,
    inputs: Union[Unset, bool] = True,
    prompts: Union[Unset, bool] = True,
    responses: Union[Unset, bool] = True,
    metrics: Union[Unset, bool] = True,
) -> Optional[Union[Any, HTTPValidationError]]:
    """Export Prompt Dataset

    Args:
        project_id (str):
        run_id (str):
        export_format (Union[Unset, LLMExportFormat]):
        include_chains (Union[Unset, bool]):  Default: True.
        columns (Union[Unset, list[str]]):
        inputs (Union[Unset, bool]):  Default: True.
        prompts (Union[Unset, bool]):  Default: True.
        responses (Union[Unset, bool]):  Default: True.
        metrics (Union[Unset, bool]):  Default: True.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, HTTPValidationError]
    """

    return (
        await asyncio_detailed(
            project_id=project_id,
            run_id=run_id,
            client=client,
            export_format=export_format,
            include_chains=include_chains,
            columns=columns,
            inputs=inputs,
            prompts=prompts,
            responses=responses,
            metrics=metrics,
        )
    ).parsed



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.node_type import NodeType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.hallucination_segment import HallucinationSegment
    from ..models.metrics import Metrics
    from ..models.prompt_row_inputs import PromptRowInputs
    from ..models.prompt_row_node_output_type_2 import PromptRowNodeOutputType2


T = TypeVar("T", bound="PromptRow")


@_attrs_define
class PromptRow:
    """
    Attributes:
        index (int):
        bleu (Union[None, Unset, float]):
        chain_id (Union[None, Unset, str]):
        chain_root_id (Union[None, Unset, str]):
        cost (Union[None, Unset, float]):
        hallucination (Union[None, Unset, float]):
        hallucination_severity (Union[Unset, int]):  Default: 0.
        has_children (Union[None, Unset, bool]):
        inputs (Union[Unset, PromptRowInputs]):
        metrics (Union[Unset, Metrics]):
        node_id (Union[None, Unset, str]):
        node_input (Union[None, Unset, str]):
        node_name (Union[None, Unset, str]):
        node_output (Union['PromptRowNodeOutputType2', None, Unset, list[str], str]):
        node_type (Union[NodeType, None, Unset]):
        prompt (Union[None, Unset, str]):
        prompt_hallucination_severity (Union[Unset, int]):  Default: 0.
        prompt_segments (Union[None, Unset, list['HallucinationSegment']]):
        response (Union[None, Unset, str]):
        response_segments (Union[Unset, list['HallucinationSegment']]):
        rouge (Union[None, Unset, float]):
        segments (Union[Unset, list['HallucinationSegment']]):
        target (Union[None, Unset, str]):
    """

    index: int
    bleu: Union[None, Unset, float] = UNSET
    chain_id: Union[None, Unset, str] = UNSET
    chain_root_id: Union[None, Unset, str] = UNSET
    cost: Union[None, Unset, float] = UNSET
    hallucination: Union[None, Unset, float] = UNSET
    hallucination_severity: Union[Unset, int] = 0
    has_children: Union[None, Unset, bool] = UNSET
    inputs: Union[Unset, "PromptRowInputs"] = UNSET
    metrics: Union[Unset, "Metrics"] = UNSET
    node_id: Union[None, Unset, str] = UNSET
    node_input: Union[None, Unset, str] = UNSET
    node_name: Union[None, Unset, str] = UNSET
    node_output: Union["PromptRowNodeOutputType2", None, Unset, list[str], str] = UNSET
    node_type: Union[NodeType, None, Unset] = UNSET
    prompt: Union[None, Unset, str] = UNSET
    prompt_hallucination_severity: Union[Unset, int] = 0
    prompt_segments: Union[None, Unset, list["HallucinationSegment"]] = UNSET
    response: Union[None, Unset, str] = UNSET
    response_segments: Union[Unset, list["HallucinationSegment"]] = UNSET
    rouge: Union[None, Unset, float] = UNSET
    segments: Union[Unset, list["HallucinationSegment"]] = UNSET
    target: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.prompt_row_node_output_type_2 import PromptRowNodeOutputType2

        index = self.index

        bleu: Union[None, Unset, float]
        if isinstance(self.bleu, Unset):
            bleu = UNSET
        else:
            bleu = self.bleu

        chain_id: Union[None, Unset, str]
        if isinstance(self.chain_id, Unset):
            chain_id = UNSET
        else:
            chain_id = self.chain_id

        chain_root_id: Union[None, Unset, str]
        if isinstance(self.chain_root_id, Unset):
            chain_root_id = UNSET
        else:
            chain_root_id = self.chain_root_id

        cost: Union[None, Unset, float]
        if isinstance(self.cost, Unset):
            cost = UNSET
        else:
            cost = self.cost

        hallucination: Union[None, Unset, float]
        if isinstance(self.hallucination, Unset):
            hallucination = UNSET
        else:
            hallucination = self.hallucination

        hallucination_severity = self.hallucination_severity

        has_children: Union[None, Unset, bool]
        if isinstance(self.has_children, Unset):
            has_children = UNSET
        else:
            has_children = self.has_children

        inputs: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.inputs, Unset):
            inputs = self.inputs.to_dict()

        metrics: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.metrics, Unset):
            metrics = self.metrics.to_dict()

        node_id: Union[None, Unset, str]
        if isinstance(self.node_id, Unset):
            node_id = UNSET
        else:
            node_id = self.node_id

        node_input: Union[None, Unset, str]
        if isinstance(self.node_input, Unset):
            node_input = UNSET
        else:
            node_input = self.node_input

        node_name: Union[None, Unset, str]
        if isinstance(self.node_name, Unset):
            node_name = UNSET
        else:
            node_name = self.node_name

        node_output: Union[None, Unset, dict[str, Any], list[str], str]
        if isinstance(self.node_output, Unset):
            node_output = UNSET
        elif isinstance(self.node_output, list):
            node_output = self.node_output

        elif isinstance(self.node_output, PromptRowNodeOutputType2):
            node_output = self.node_output.to_dict()
        else:
            node_output = self.node_output

        node_type: Union[None, Unset, str]
        if isinstance(self.node_type, Unset):
            node_type = UNSET
        elif isinstance(self.node_type, NodeType):
            node_type = self.node_type.value
        else:
            node_type = self.node_type

        prompt: Union[None, Unset, str]
        if isinstance(self.prompt, Unset):
            prompt = UNSET
        else:
            prompt = self.prompt

        prompt_hallucination_severity = self.prompt_hallucination_severity

        prompt_segments: Union[None, Unset, list[dict[str, Any]]]
        if isinstance(self.prompt_segments, Unset):
            prompt_segments = UNSET
        elif isinstance(self.prompt_segments, list):
            prompt_segments = []
            for prompt_segments_type_0_item_data in self.prompt_segments:
                prompt_segments_type_0_item = prompt_segments_type_0_item_data.to_dict()
                prompt_segments.append(prompt_segments_type_0_item)

        else:
            prompt_segments = self.prompt_segments

        response: Union[None, Unset, str]
        if isinstance(self.response, Unset):
            response = UNSET
        else:
            response = self.response

        response_segments: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.response_segments, Unset):
            response_segments = []
            for response_segments_item_data in self.response_segments:
                response_segments_item = response_segments_item_data.to_dict()
                response_segments.append(response_segments_item)

        rouge: Union[None, Unset, float]
        if isinstance(self.rouge, Unset):
            rouge = UNSET
        else:
            rouge = self.rouge

        segments: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.segments, Unset):
            segments = []
            for segments_item_data in self.segments:
                segments_item = segments_item_data.to_dict()
                segments.append(segments_item)

        target: Union[None, Unset, str]
        if isinstance(self.target, Unset):
            target = UNSET
        else:
            target = self.target

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"index": index})
        if bleu is not UNSET:
            field_dict["bleu"] = bleu
        if chain_id is not UNSET:
            field_dict["chain_id"] = chain_id
        if chain_root_id is not UNSET:
            field_dict["chain_root_id"] = chain_root_id
        if cost is not UNSET:
            field_dict["cost"] = cost
        if hallucination is not UNSET:
            field_dict["hallucination"] = hallucination
        if hallucination_severity is not UNSET:
            field_dict["hallucination_severity"] = hallucination_severity
        if has_children is not UNSET:
            field_dict["has_children"] = has_children
        if inputs is not UNSET:
            field_dict["inputs"] = inputs
        if metrics is not UNSET:
            field_dict["metrics"] = metrics
        if node_id is not UNSET:
            field_dict["node_id"] = node_id
        if node_input is not UNSET:
            field_dict["node_input"] = node_input
        if node_name is not UNSET:
            field_dict["node_name"] = node_name
        if node_output is not UNSET:
            field_dict["node_output"] = node_output
        if node_type is not UNSET:
            field_dict["node_type"] = node_type
        if prompt is not UNSET:
            field_dict["prompt"] = prompt
        if prompt_hallucination_severity is not UNSET:
            field_dict["prompt_hallucination_severity"] = prompt_hallucination_severity
        if prompt_segments is not UNSET:
            field_dict["prompt_segments"] = prompt_segments
        if response is not UNSET:
            field_dict["response"] = response
        if response_segments is not UNSET:
            field_dict["response_segments"] = response_segments
        if rouge is not UNSET:
            field_dict["rouge"] = rouge
        if segments is not UNSET:
            field_dict["segments"] = segments
        if target is not UNSET:
            field_dict["target"] = target

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.hallucination_segment import HallucinationSegment
        from ..models.metrics import Metrics
        from ..models.prompt_row_inputs import PromptRowInputs
        from ..models.prompt_row_node_output_type_2 import PromptRowNodeOutputType2

        d = src_dict.copy()
        index = d.pop("index")

        def _parse_bleu(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        bleu = _parse_bleu(d.pop("bleu", UNSET))

        def _parse_chain_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        chain_id = _parse_chain_id(d.pop("chain_id", UNSET))

        def _parse_chain_root_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        chain_root_id = _parse_chain_root_id(d.pop("chain_root_id", UNSET))

        def _parse_cost(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        cost = _parse_cost(d.pop("cost", UNSET))

        def _parse_hallucination(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        hallucination = _parse_hallucination(d.pop("hallucination", UNSET))

        hallucination_severity = d.pop("hallucination_severity", UNSET)

        def _parse_has_children(data: object) -> Union[None, Unset, bool]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool], data)

        has_children = _parse_has_children(d.pop("has_children", UNSET))

        _inputs = d.pop("inputs", UNSET)
        inputs: Union[Unset, PromptRowInputs]
        if isinstance(_inputs, Unset):
            inputs = UNSET
        else:
            inputs = PromptRowInputs.from_dict(_inputs)

        _metrics = d.pop("metrics", UNSET)
        metrics: Union[Unset, Metrics]
        if isinstance(_metrics, Unset):
            metrics = UNSET
        else:
            metrics = Metrics.from_dict(_metrics)

        def _parse_node_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        node_id = _parse_node_id(d.pop("node_id", UNSET))

        def _parse_node_input(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        node_input = _parse_node_input(d.pop("node_input", UNSET))

        def _parse_node_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        node_name = _parse_node_name(d.pop("node_name", UNSET))

        def _parse_node_output(data: object) -> Union["PromptRowNodeOutputType2", None, Unset, list[str], str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                node_output_type_1 = cast(list[str], data)

                return node_output_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                node_output_type_2 = PromptRowNodeOutputType2.from_dict(data)

                return node_output_type_2
            except:  # noqa: E722
                pass
            return cast(Union["PromptRowNodeOutputType2", None, Unset, list[str], str], data)

        node_output = _parse_node_output(d.pop("node_output", UNSET))

        def _parse_node_type(data: object) -> Union[NodeType, None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                node_type_type_0 = NodeType(data)

                return node_type_type_0
            except:  # noqa: E722
                pass
            return cast(Union[NodeType, None, Unset], data)

        node_type = _parse_node_type(d.pop("node_type", UNSET))

        def _parse_prompt(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        prompt = _parse_prompt(d.pop("prompt", UNSET))

        prompt_hallucination_severity = d.pop("prompt_hallucination_severity", UNSET)

        def _parse_prompt_segments(data: object) -> Union[None, Unset, list["HallucinationSegment"]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                prompt_segments_type_0 = []
                _prompt_segments_type_0 = data
                for prompt_segments_type_0_item_data in _prompt_segments_type_0:
                    prompt_segments_type_0_item = HallucinationSegment.from_dict(prompt_segments_type_0_item_data)

                    prompt_segments_type_0.append(prompt_segments_type_0_item)

                return prompt_segments_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list["HallucinationSegment"]], data)

        prompt_segments = _parse_prompt_segments(d.pop("prompt_segments", UNSET))

        def _parse_response(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        response = _parse_response(d.pop("response", UNSET))

        response_segments = []
        _response_segments = d.pop("response_segments", UNSET)
        for response_segments_item_data in _response_segments or []:
            response_segments_item = HallucinationSegment.from_dict(response_segments_item_data)

            response_segments.append(response_segments_item)

        def _parse_rouge(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        rouge = _parse_rouge(d.pop("rouge", UNSET))

        segments = []
        _segments = d.pop("segments", UNSET)
        for segments_item_data in _segments or []:
            segments_item = HallucinationSegment.from_dict(segments_item_data)

            segments.append(segments_item)

        def _parse_target(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        target = _parse_target(d.pop("target", UNSET))

        prompt_row = cls(
            index=index,
            bleu=bleu,
            chain_id=chain_id,
            chain_root_id=chain_root_id,
            cost=cost,
            hallucination=hallucination,
            hallucination_severity=hallucination_severity,
            has_children=has_children,
            inputs=inputs,
            metrics=metrics,
            node_id=node_id,
            node_input=node_input,
            node_name=node_name,
            node_output=node_output,
            node_type=node_type,
            prompt=prompt,
            prompt_hallucination_severity=prompt_hallucination_severity,
            prompt_segments=prompt_segments,
            response=response,
            response_segments=response_segments,
            rouge=rouge,
            segments=segments,
            target=target,
        )

        prompt_row.additional_properties = d
        return prompt_row

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.put_prompt_score_request_aggregates_type_0 import PutPromptScoreRequestAggregatesType0


T = TypeVar("T", bound="PutPromptScoreRequest")


@_attrs_define
class PutPromptScoreRequest:
    """
    Attributes:
        aggregates (Union['PutPromptScoreRequestAggregatesType0', None, Unset]):
        indices (Union[None, Unset, list[int]]):
        metric_name (Union[None, Unset, str]):
        registered_scorer_id (Union[None, Unset, str]):
        scores (Union[None, Unset, list[Any]]):
    """

    aggregates: Union["PutPromptScoreRequestAggregatesType0", None, Unset] = UNSET
    indices: Union[None, Unset, list[int]] = UNSET
    metric_name: Union[None, Unset, str] = UNSET
    registered_scorer_id: Union[None, Unset, str] = UNSET
    scores: Union[None, Unset, list[Any]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.put_prompt_score_request_aggregates_type_0 import PutPromptScoreRequestAggregatesType0

        aggregates: Union[None, Unset, dict[str, Any]]
        if isinstance(self.aggregates, Unset):
            aggregates = UNSET
        elif isinstance(self.aggregates, PutPromptScoreRequestAggregatesType0):
            aggregates = self.aggregates.to_dict()
        else:
            aggregates = self.aggregates

        indices: Union[None, Unset, list[int]]
        if isinstance(self.indices, Unset):
            indices = UNSET
        elif isinstance(self.indices, list):
            indices = self.indices

        else:
            indices = self.indices

        metric_name: Union[None, Unset, str]
        if isinstance(self.metric_name, Unset):
            metric_name = UNSET
        else:
            metric_name = self.metric_name

        registered_scorer_id: Union[None, Unset, str]
        if isinstance(self.registered_scorer_id, Unset):
            registered_scorer_id = UNSET
        else:
            registered_scorer_id = self.registered_scorer_id

        scores: Union[None, Unset, list[Any]]
        if isinstance(self.scores, Unset):
            scores = UNSET
        elif isinstance(self.scores, list):
            scores = self.scores

        else:
            scores = self.scores

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if aggregates is not UNSET:
            field_dict["aggregates"] = aggregates
        if indices is not UNSET:
            field_dict["indices"] = indices
        if metric_name is not UNSET:
            field_dict["metric_name"] = metric_name
        if registered_scorer_id is not UNSET:
            field_dict["registered_scorer_id"] = registered_scorer_id
        if scores is not UNSET:
            field_dict["scores"] = scores

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.put_prompt_score_request_aggregates_type_0 import PutPromptScoreRequestAggregatesType0

        d = src_dict.copy()

        def _parse_aggregates(data: object) -> Union["PutPromptScoreRequestAggregatesType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                aggregates_type_0 = PutPromptScoreRequestAggregatesType0.from_dict(data)

                return aggregates_type_0
            except:  # noqa: E722
                pass
            return cast(Union["PutPromptScoreRequestAggregatesType0", None, Unset], data)

        aggregates = _parse_aggregates(d.pop("aggregates", UNSET))

        def _parse_indices(data: object) -> Union[None, Unset, list[int]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                indices_type_0 = cast(list[int], data)

                return indices_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[int]], data)

        indices = _parse_indices(d.pop("indices", UNSET))

        def _parse_metric_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        metric_name = _parse_metric_name(d.pop("metric_name", UNSET))

        def _parse_registered_scorer_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        registered_scorer_id = _parse_registered_scorer_id(d.pop("registered_scorer_id", UNSET))

        def _parse_scores(data: object) -> Union[None, Unset, list[Any]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                scores_type_0 = cast(list[Any], data)

                return scores_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[Any]], data)

        scores = _parse_scores(d.pop("scores", UNSET))

        put_prompt_score_request = cls(
            aggregates=aggregates,
            indices=indices,
            metric_name=metric_name,
            registered_scorer_id=registered_scorer_id,
            scores=scores,
        )

        put_prompt_score_request.additional_properties = d
        return put_prompt_score_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define

from ..models.model_kind import ModelKind

if TYPE_CHECKING:
    from ..models.upload_model_parameters import UploadModelParameters


T = TypeVar("T", bound="UploadModel")


@_attrs_define
class UploadModel:
    """
    Attributes:
        kind (ModelKind):
        parameters (UploadModelParameters):
    """

    kind: ModelKind
    parameters: "UploadModelParameters"

    def to_dict(self) -> dict[str, Any]:
        kind = self.kind.value

        parameters = self.parameters.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update({"kind": kind, "parameters": parameters})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.upload_model_parameters import UploadModelParameters

        d = src_dict.copy()
        kind = ModelKind(d.pop("kind"))

        parameters = UploadModelParameters.from_dict(d.pop("parameters"))

        upload_model = cls(kind=kind, parameters=parameters)

        return upload_model



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.anthropic_integration import AnthropicIntegration
from ...models.aws_bedrock_integration import AwsBedrockIntegration
from ...models.aws_sage_maker_integration import AwsSageMakerIntegration
from ...models.azure_integration import AzureIntegration
from ...models.databricks_integration import DatabricksIntegration
from ...models.http_validation_error import HTTPValidationError
from ...models.integration_name import IntegrationName
from ...models.label_studio_integration import LabelStudioIntegration
from ...models.mistral_integration import MistralIntegration
from ...models.open_ai_integration import OpenAIIntegration
from ...models.vertex_ai_integration import VertexAIIntegration
from ...models.writer_integration import WriterIntegration
from ...types import Response


def _get_kwargs(name: IntegrationName) -> dict[str, Any]:
    _kwargs: dict[str, Any] = {"method": "get", "url": f"/integrations/{name}"}

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[
    Union[
        HTTPValidationError,
        Union[
            "AnthropicIntegration",
            "AwsBedrockIntegration",
            "AwsSageMakerIntegration",
            "AzureIntegration",
            "DatabricksIntegration",
            "LabelStudioIntegration",
            "MistralIntegration",
            "OpenAIIntegration",
            "VertexAIIntegration",
            "WriterIntegration",
        ],
    ]
]:
    if response.status_code == 200:

        def _parse_response_200(
            data: object,
        ) -> Union[
            "AnthropicIntegration",
            "AwsBedrockIntegration",
            "AwsSageMakerIntegration",
            "AzureIntegration",
            "DatabricksIntegration",
            "LabelStudioIntegration",
            "MistralIntegration",
            "OpenAIIntegration",
            "VertexAIIntegration",
            "WriterIntegration",
        ]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200_type_0 = AwsBedrockIntegration.from_dict(data)

                return response_200_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200_type_1 = AwsSageMakerIntegration.from_dict(data)

                return response_200_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200_type_2 = AzureIntegration.from_dict(data)

                return response_200_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200_type_3 = DatabricksIntegration.from_dict(data)

                return response_200_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200_type_4 = LabelStudioIntegration.from_dict(data)

                return response_200_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200_type_5 = OpenAIIntegration.from_dict(data)

                return response_200_type_5
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200_type_6 = VertexAIIntegration.from_dict(data)

                return response_200_type_6
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200_type_7 = WriterIntegration.from_dict(data)

                return response_200_type_7
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200_type_8 = AnthropicIntegration.from_dict(data)

                return response_200_type_8
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            response_200_type_9 = MistralIntegration.from_dict(data)

            return response_200_type_9

        response_200 = _parse_response_200(response.json())

        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[
    Union[
        HTTPValidationError,
        Union[
            "AnthropicIntegration",
            "AwsBedrockIntegration",
            "AwsSageMakerIntegration",
            "AzureIntegration",
            "DatabricksIntegration",
            "LabelStudioIntegration",
            "MistralIntegration",
            "OpenAIIntegration",
            "VertexAIIntegration",
            "WriterIntegration",
        ],
    ]
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    name: IntegrationName, *, client: AuthenticatedClient
) -> Response[
    Union[
        HTTPValidationError,
        Union[
            "AnthropicIntegration",
            "AwsBedrockIntegration",
            "AwsSageMakerIntegration",
            "AzureIntegration",
            "DatabricksIntegration",
            "LabelStudioIntegration",
            "MistralIntegration",
            "OpenAIIntegration",
            "VertexAIIntegration",
            "WriterIntegration",
        ],
    ]
]:
    """Get Integration

     Gets the integration data formatted for the specified integration.

    Args:
        name (IntegrationName):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, Union['AnthropicIntegration', 'AwsBedrockIntegration', 'AwsSageMakerIntegration', 'AzureIntegration', 'DatabricksIntegration', 'LabelStudioIntegration', 'MistralIntegration', 'OpenAIIntegration', 'VertexAIIntegration', 'WriterIntegration']]]
    """

    kwargs = _get_kwargs(name=name)

    response = client.get_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


def sync(
    name: IntegrationName, *, client: AuthenticatedClient
) -> Optional[
    Union[
        HTTPValidationError,
        Union[
            "AnthropicIntegration",
            "AwsBedrockIntegration",
            "AwsSageMakerIntegration",
            "AzureIntegration",
            "DatabricksIntegration",
            "LabelStudioIntegration",
            "MistralIntegration",
            "OpenAIIntegration",
            "VertexAIIntegration",
            "WriterIntegration",
        ],
    ]
]:
    """Get Integration

     Gets the integration data formatted for the specified integration.

    Args:
        name (IntegrationName):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, Union['AnthropicIntegration', 'AwsBedrockIntegration', 'AwsSageMakerIntegration', 'AzureIntegration', 'DatabricksIntegration', 'LabelStudioIntegration', 'MistralIntegration', 'OpenAIIntegration', 'VertexAIIntegration', 'WriterIntegration']]
    """

    return sync_detailed(name=name, client=client).parsed


async def asyncio_detailed(
    name: IntegrationName, *, client: AuthenticatedClient
) -> Response[
    Union[
        HTTPValidationError,
        Union[
            "AnthropicIntegration",
            "AwsBedrockIntegration",
            "AwsSageMakerIntegration",
            "AzureIntegration",
            "DatabricksIntegration",
            "LabelStudioIntegration",
            "MistralIntegration",
            "OpenAIIntegration",
            "VertexAIIntegration",
            "WriterIntegration",
        ],
    ]
]:
    """Get Integration

     Gets the integration data formatted for the specified integration.

    Args:
        name (IntegrationName):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, Union['AnthropicIntegration', 'AwsBedrockIntegration', 'AwsSageMakerIntegration', 'AzureIntegration', 'DatabricksIntegration', 'LabelStudioIntegration', 'MistralIntegration', 'OpenAIIntegration', 'VertexAIIntegration', 'WriterIntegration']]]
    """

    kwargs = _get_kwargs(name=name)

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    name: IntegrationName, *, client: AuthenticatedClient
) -> Optional[
    Union[
        HTTPValidationError,
        Union[
            "AnthropicIntegration",
            "AwsBedrockIntegration",
            "AwsSageMakerIntegration",
            "AzureIntegration",
            "DatabricksIntegration",
            "LabelStudioIntegration",
            "MistralIntegration",
            "OpenAIIntegration",
            "VertexAIIntegration",
            "WriterIntegration",
        ],
    ]
]:
    """Get Integration

     Gets the integration data formatted for the specified integration.

    Args:
        name (IntegrationName):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, Union['AnthropicIntegration', 'AwsBedrockIntegration', 'AwsSageMakerIntegration', 'AzureIntegration', 'DatabricksIntegration', 'LabelStudioIntegration', 'MistralIntegration', 'OpenAIIntegration', 'VertexAIIntegration', 'WriterIntegration']]
    """

    return (await asyncio_detailed(name=name, client=client)).parsed



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.job_db import JobDB
from ...types import Response


def _get_kwargs(job_id: str) -> dict[str, Any]:
    _kwargs: dict[str, Any] = {"method": "get", "url": f"/jobs/{job_id}"}

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[HTTPValidationError, JobDB]]:
    if response.status_code == 200:
        response_200 = JobDB.from_dict(response.json())

        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[HTTPValidationError, JobDB]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(job_id: str, *, client: AuthenticatedClient) -> Response[Union[HTTPValidationError, JobDB]]:
    """Get Job

     Get a job by id.

    Args:
        job_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, JobDB]]
    """

    kwargs = _get_kwargs(job_id=job_id)

    response = client.get_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


def sync(job_id: str, *, client: AuthenticatedClient) -> Optional[Union[HTTPValidationError, JobDB]]:
    """Get Job

     Get a job by id.

    Args:
        job_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, JobDB]
    """

    return sync_detailed(job_id=job_id, client=client).parsed


async def asyncio_detailed(job_id: str, *, client: AuthenticatedClient) -> Response[Union[HTTPValidationError, JobDB]]:
    """Get Job

     Get a job by id.

    Args:
        job_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, JobDB]]
    """

    kwargs = _get_kwargs(job_id=job_id)

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(job_id: str, *, client: AuthenticatedClient) -> Optional[Union[HTTPValidationError, JobDB]]:
    """Get Job

     Get a job by id.

    Args:
        job_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, JobDB]
    """

    return (await asyncio_detailed(job_id=job_id, client=client)).parsed



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.split import Split
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.filter_params import FilterParams


T = TypeVar("T", bound="LabelStudioExportRequest")


@_attrs_define
class LabelStudioExportRequest:
    """Schema for requesting data to be sent to labelstudio.

    Coming soon!

        Attributes:
            project_id (str):
            project_name (str):
            run_id (str):
            split (Split):
            compare_to (Union[None, Split, Unset]):
            filter_params (Union[Unset, FilterParams]):
            inference_name (Union[None, Unset, str]):
            labels (Union[None, Unset, list[str]]):
            map_threshold (Union[Unset, float]):  Default: 0.5.
            task (Union[None, Unset, str]):
            workspace (Union[None, Unset, str]):
    """

    project_id: str
    project_name: str
    run_id: str
    split: Split
    compare_to: Union[None, Split, Unset] = UNSET
    filter_params: Union[Unset, "FilterParams"] = UNSET
    inference_name: Union[None, Unset, str] = UNSET
    labels: Union[None, Unset, list[str]] = UNSET
    map_threshold: Union[Unset, float] = 0.5
    task: Union[None, Unset, str] = UNSET
    workspace: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        project_id = self.project_id

        project_name = self.project_name

        run_id = self.run_id

        split = self.split.value

        compare_to: Union[None, Unset, str]
        if isinstance(self.compare_to, Unset):
            compare_to = UNSET
        elif isinstance(self.compare_to, Split):
            compare_to = self.compare_to.value
        else:
            compare_to = self.compare_to

        filter_params: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.filter_params, Unset):
            filter_params = self.filter_params.to_dict()

        inference_name: Union[None, Unset, str]
        if isinstance(self.inference_name, Unset):
            inference_name = UNSET
        else:
            inference_name = self.inference_name

        labels: Union[None, Unset, list[str]]
        if isinstance(self.labels, Unset):
            labels = UNSET
        elif isinstance(self.labels, list):
            labels = self.labels

        else:
            labels = self.labels

        map_threshold = self.map_threshold

        task: Union[None, Unset, str]
        if isinstance(self.task, Unset):
            task = UNSET
        else:
            task = self.task

        workspace: Union[None, Unset, str]
        if isinstance(self.workspace, Unset):
            workspace = UNSET
        else:
            workspace = self.workspace

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"project_id": project_id, "project_name": project_name, "run_id": run_id, "split": split})
        if compare_to is not UNSET:
            field_dict["compare_to"] = compare_to
        if filter_params is not UNSET:
            field_dict["filter_params"] = filter_params
        if inference_name is not UNSET:
            field_dict["inference_name"] = inference_name
        if labels is not UNSET:
            field_dict["labels"] = labels
        if map_threshold is not UNSET:
            field_dict["map_threshold"] = map_threshold
        if task is not UNSET:
            field_dict["task"] = task
        if workspace is not UNSET:
            field_dict["workspace"] = workspace

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.filter_params import FilterParams

        d = src_dict.copy()
        project_id = d.pop("project_id")

        project_name = d.pop("project_name")

        run_id = d.pop("run_id")

        split = Split(d.pop("split"))

        def _parse_compare_to(data: object) -> Union[None, Split, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                compare_to_type_0 = Split(data)

                return compare_to_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Split, Unset], data)

        compare_to = _parse_compare_to(d.pop("compare_to", UNSET))

        _filter_params = d.pop("filter_params", UNSET)
        filter_params: Union[Unset, FilterParams]
        if isinstance(_filter_params, Unset):
            filter_params = UNSET
        else:
            filter_params = FilterParams.from_dict(_filter_params)

        def _parse_inference_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        inference_name = _parse_inference_name(d.pop("inference_name", UNSET))

        def _parse_labels(data: object) -> Union[None, Unset, list[str]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                labels_type_0 = cast(list[str], data)

                return labels_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[str]], data)

        labels = _parse_labels(d.pop("labels", UNSET))

        map_threshold = d.pop("map_threshold", UNSET)

        def _parse_task(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        task = _parse_task(d.pop("task", UNSET))

        def _parse_workspace(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        workspace = _parse_workspace(d.pop("workspace", UNSET))

        label_studio_export_request = cls(
            project_id=project_id,
            project_name=project_name,
            run_id=run_id,
            split=split,
            compare_to=compare_to,
            filter_params=filter_params,
            inference_name=inference_name,
            labels=labels,
            map_threshold=map_threshold,
            task=task,
            workspace=workspace,
        )

        label_studio_export_request.additional_properties = d
        return label_studio_export_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.permission import Permission


T = TypeVar("T", bound="OrganizationDB")


@_attrs_define
class OrganizationDB:
    """
    Attributes:
        created_at (datetime.datetime):
        id (str):
        name (str):
        updated_at (datetime.datetime):
        permissions (Union[Unset, list['Permission']]):
    """

    created_at: datetime.datetime
    id: str
    name: str
    updated_at: datetime.datetime
    permissions: Union[Unset, list["Permission"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        created_at = self.created_at.isoformat()

        id = self.id

        name = self.name

        updated_at = self.updated_at.isoformat()

        permissions: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.permissions, Unset):
            permissions = []
            for permissions_item_data in self.permissions:
                permissions_item = permissions_item_data.to_dict()
                permissions.append(permissions_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"created_at": created_at, "id": id, "name": name, "updated_at": updated_at})
        if permissions is not UNSET:
            field_dict["permissions"] = permissions

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.permission import Permission

        d = src_dict.copy()
        created_at = isoparse(d.pop("created_at"))

        id = d.pop("id")

        name = d.pop("name")

        updated_at = isoparse(d.pop("updated_at"))

        permissions = []
        _permissions = d.pop("permissions", UNSET)
        for permissions_item_data in _permissions or []:
            permissions_item = Permission.from_dict(permissions_item_data)

            permissions.append(permissions_item)

        organization_db = cls(created_at=created_at, id=id, name=name, updated_at=updated_at, permissions=permissions)

        organization_db.additional_properties = d
        return organization_db

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.evaluate_run_request import EvaluateRunRequest
from ...models.evaluate_run_response import EvaluateRunResponse
from ...models.http_validation_error import HTTPValidationError
from ...types import Response


def _get_kwargs(*, body: EvaluateRunRequest) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {"method": "post", "url": "/v1/evaluate/runs"}

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[EvaluateRunResponse, HTTPValidationError]]:
    if response.status_code == 200:
        response_200 = EvaluateRunResponse.from_dict(response.json())

        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[EvaluateRunResponse, HTTPValidationError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *, client: AuthenticatedClient, body: EvaluateRunRequest
) -> Response[Union[EvaluateRunResponse, HTTPValidationError]]:
    """Create Workflows Run

     Create a new Evaluate run with workflows.

    Use this endpoint to create a new Evaluate run with workflows. The request body should contain the
    `workflows` to be ingested and evaluated.

    Additionally, specify the `project_id` or `project_name` to which the workflows should be ingested.
    If the project does not exist, it will be created. If the project exists, the workflows will be
    logged to it. If both `project_id` and `project_name` are provided, `project_id` will take
    precedence. The `run_name` is optional and will be auto-generated (timestamp-based) if not provided.

    The body is also expected to include the configuration for the scorers to be used in the evaluation.
    This configuration will be used to evaluate the workflows and generate the results.

    Args:
        body (EvaluateRunRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[EvaluateRunResponse, HTTPValidationError]]
    """

    kwargs = _get_kwargs(body=body)

    response = client.get_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


def sync(
    *, client: AuthenticatedClient, body: EvaluateRunRequest
) -> Optional[Union[EvaluateRunResponse, HTTPValidationError]]:
    """Create Workflows Run

     Create a new Evaluate run with workflows.

    Use this endpoint to create a new Evaluate run with workflows. The request body should contain the
    `workflows` to be ingested and evaluated.

    Additionally, specify the `project_id` or `project_name` to which the workflows should be ingested.
    If the project does not exist, it will be created. If the project exists, the workflows will be
    logged to it. If both `project_id` and `project_name` are provided, `project_id` will take
    precedence. The `run_name` is optional and will be auto-generated (timestamp-based) if not provided.

    The body is also expected to include the configuration for the scorers to be used in the evaluation.
    This configuration will be used to evaluate the workflows and generate the results.

    Args:
        body (EvaluateRunRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[EvaluateRunResponse, HTTPValidationError]
    """

    return sync_detailed(client=client, body=body).parsed


async def asyncio_detailed(
    *, client: AuthenticatedClient, body: EvaluateRunRequest
) -> Response[Union[EvaluateRunResponse, HTTPValidationError]]:
    """Create Workflows Run

     Create a new Evaluate run with workflows.

    Use this endpoint to create a new Evaluate run with workflows. The request body should contain the
    `workflows` to be ingested and evaluated.

    Additionally, specify the `project_id` or `project_name` to which the workflows should be ingested.
    If the project does not exist, it will be created. If the project exists, the workflows will be
    logged to it. If both `project_id` and `project_name` are provided, `project_id` will take
    precedence. The `run_name` is optional and will be auto-generated (timestamp-based) if not provided.

    The body is also expected to include the configuration for the scorers to be used in the evaluation.
    This configuration will be used to evaluate the workflows and generate the results.

    Args:
        body (EvaluateRunRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[EvaluateRunResponse, HTTPValidationError]]
    """

    kwargs = _get_kwargs(body=body)

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *, client: AuthenticatedClient, body: EvaluateRunRequest
) -> Optional[Union[EvaluateRunResponse, HTTPValidationError]]:
    """Create Workflows Run

     Create a new Evaluate run with workflows.

    Use this endpoint to create a new Evaluate run with workflows. The request body should contain the
    `workflows` to be ingested and evaluated.

    Additionally, specify the `project_id` or `project_name` to which the workflows should be ingested.
    If the project does not exist, it will be created. If the project exists, the workflows will be
    logged to it. If both `project_id` and `project_name` are provided, `project_id` will take
    precedence. The `run_name` is optional and will be auto-generated (timestamp-based) if not provided.

    The body is also expected to include the configuration for the scorers to be used in the evaluation.
    This configuration will be used to evaluate the workflows and generate the results.

    Args:
        body (EvaluateRunRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[EvaluateRunResponse, HTTPValidationError]
    """

    return (await asyncio_detailed(client=client, body=body)).parsed



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.few_shot_example import FewShotExample


T = TypeVar("T", bound="AutoGenResult")


@_attrs_define
class AutoGenResult:
    """
    Attributes:
        template (str): Chainpoll prompt template.
        explanation_field_name (Union[Unset, str]): Field name to look for in the chainpoll response, for the
            explanation. Default: 'explanation'.
        metric_description (Union[None, Unset, str]): Description of what the metric should do.
        metric_few_shot_examples (Union[Unset, list['FewShotExample']]): Few-shot examples for the metric.
        metric_system_prompt (Union[None, Unset, str]): System prompt for the metric.
        value_field_name (Union[Unset, str]): Field name to look for in the chainpoll response, for the rating. Default:
            'rating'.
    """

    template: str
    explanation_field_name: Union[Unset, str] = "explanation"
    metric_description: Union[None, Unset, str] = UNSET
    metric_few_shot_examples: Union[Unset, list["FewShotExample"]] = UNSET
    metric_system_prompt: Union[None, Unset, str] = UNSET
    value_field_name: Union[Unset, str] = "rating"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        template = self.template

        explanation_field_name = self.explanation_field_name

        metric_description: Union[None, Unset, str]
        if isinstance(self.metric_description, Unset):
            metric_description = UNSET
        else:
            metric_description = self.metric_description

        metric_few_shot_examples: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.metric_few_shot_examples, Unset):
            metric_few_shot_examples = []
            for metric_few_shot_examples_item_data in self.metric_few_shot_examples:
                metric_few_shot_examples_item = metric_few_shot_examples_item_data.to_dict()
                metric_few_shot_examples.append(metric_few_shot_examples_item)

        metric_system_prompt: Union[None, Unset, str]
        if isinstance(self.metric_system_prompt, Unset):
            metric_system_prompt = UNSET
        else:
            metric_system_prompt = self.metric_system_prompt

        value_field_name = self.value_field_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"template": template})
        if explanation_field_name is not UNSET:
            field_dict["explanation_field_name"] = explanation_field_name
        if metric_description is not UNSET:
            field_dict["metric_description"] = metric_description
        if metric_few_shot_examples is not UNSET:
            field_dict["metric_few_shot_examples"] = metric_few_shot_examples
        if metric_system_prompt is not UNSET:
            field_dict["metric_system_prompt"] = metric_system_prompt
        if value_field_name is not UNSET:
            field_dict["value_field_name"] = value_field_name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.few_shot_example import FewShotExample

        d = src_dict.copy()
        template = d.pop("template")

        explanation_field_name = d.pop("explanation_field_name", UNSET)

        def _parse_metric_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        metric_description = _parse_metric_description(d.pop("metric_description", UNSET))

        metric_few_shot_examples = []
        _metric_few_shot_examples = d.pop("metric_few_shot_examples", UNSET)
        for metric_few_shot_examples_item_data in _metric_few_shot_examples or []:
            metric_few_shot_examples_item = FewShotExample.from_dict(metric_few_shot_examples_item_data)

            metric_few_shot_examples.append(metric_few_shot_examples_item)

        def _parse_metric_system_prompt(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        metric_system_prompt = _parse_metric_system_prompt(d.pop("metric_system_prompt", UNSET))

        value_field_name = d.pop("value_field_name", UNSET)

        auto_gen_result = cls(
            template=template,
            explanation_field_name=explanation_field_name,
            metric_description=metric_description,
            metric_few_shot_examples=metric_few_shot_examples,
            metric_system_prompt=metric_system_prompt,
            value_field_name=value_field_name,
        )

        auto_gen_result.additional_properties = d
        return auto_gen_result

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="TargetLabels")


@_attrs_define
class TargetLabels:
    """Structure for each target label in a co-occurence response.

    Attributes:
        co_occurrence (float):
        label (str):
    """

    co_occurrence: float
    label: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        co_occurrence = self.co_occurrence

        label = self.label

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"co_occurrence": co_occurrence, "label": label})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        co_occurrence = d.pop("co_occurrence")

        label = d.pop("label")

        target_labels = cls(co_occurrence=co_occurrence, label=label)

        target_labels.additional_properties = d
        return target_labels

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.prompt_sample_i_ds_sample_indices import PromptSampleIDsSampleIndices


T = TypeVar("T", bound="PromptSampleIDs")


@_attrs_define
class PromptSampleIDs:
    """
    Attributes:
        sample_indices (PromptSampleIDsSampleIndices):
        total_samples (int):
    """

    sample_indices: "PromptSampleIDsSampleIndices"
    total_samples: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        sample_indices = self.sample_indices.to_dict()

        total_samples = self.total_samples

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"sample_indices": sample_indices, "total_samples": total_samples})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.prompt_sample_i_ds_sample_indices import PromptSampleIDsSampleIndices

        d = src_dict.copy()
        sample_indices = PromptSampleIDsSampleIndices.from_dict(d.pop("sample_indices"))

        total_samples = d.pop("total_samples")

        prompt_sample_i_ds = cls(sample_indices=sample_indices, total_samples=total_samples)

        prompt_sample_i_ds.additional_properties = d
        return prompt_sample_i_ds

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.collaborator_role import CollaboratorRole
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.permission import Permission


T = TypeVar("T", bound="UserCollaborator")


@_attrs_define
class UserCollaborator:
    """
    Attributes:
        created_at (datetime.datetime):
        email (str):
        first_name (Union[None, str]):
        id (str):
        last_name (Union[None, str]):
        role (CollaboratorRole):
        user_id (str):
        permissions (Union[Unset, list['Permission']]):
    """

    created_at: datetime.datetime
    email: str
    first_name: Union[None, str]
    id: str
    last_name: Union[None, str]
    role: CollaboratorRole
    user_id: str
    permissions: Union[Unset, list["Permission"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        created_at = self.created_at.isoformat()

        email = self.email

        first_name: Union[None, str]
        first_name = self.first_name

        id = self.id

        last_name: Union[None, str]
        last_name = self.last_name

        role = self.role.value

        user_id = self.user_id

        permissions: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.permissions, Unset):
            permissions = []
            for permissions_item_data in self.permissions:
                permissions_item = permissions_item_data.to_dict()
                permissions.append(permissions_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "created_at": created_at,
                "email": email,
                "first_name": first_name,
                "id": id,
                "last_name": last_name,
                "role": role,
                "user_id": user_id,
            }
        )
        if permissions is not UNSET:
            field_dict["permissions"] = permissions

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.permission import Permission

        d = src_dict.copy()
        created_at = isoparse(d.pop("created_at"))

        email = d.pop("email")

        def _parse_first_name(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        first_name = _parse_first_name(d.pop("first_name"))

        id = d.pop("id")

        def _parse_last_name(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        last_name = _parse_last_name(d.pop("last_name"))

        role = CollaboratorRole(d.pop("role"))

        user_id = d.pop("user_id")

        permissions = []
        _permissions = d.pop("permissions", UNSET)
        for permissions_item_data in _permissions or []:
            permissions_item = Permission.from_dict(permissions_item_data)

            permissions.append(permissions_item)

        user_collaborator = cls(
            created_at=created_at,
            email=email,
            first_name=first_name,
            id=id,
            last_name=last_name,
            role=role,
            user_id=user_id,
            permissions=permissions,
        )

        user_collaborator.additional_properties = d
        return user_collaborator

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
Galileo wrapper for OpenAI that automatically logs prompts and responses.

This module provides a drop-in replacement for the OpenAI library that automatically
logs all prompts, responses, and related metadata to Galileo. It works by intercepting
calls to the OpenAI API and logging them using the Galileo logging system.

Note that the original OpenAI package is still required as a project dependency to use this wrapper.

Examples
--------
# Import the wrapped OpenAI client instead of the original
from galileo.openai import openai

# Use it exactly as you would use the regular OpenAI client
response = openai.chat.completions.create(
    model="gpt-4o",
    messages=[
        {"role": "system", "content": "You are a helpful assistant."},
        {"role": "user", "content": "Tell me about the solar system."}
    ]
)

# All prompts and responses are automatically logged to Galileo
print(response.choices[0].message.content)

# You can also use it with the galileo_context for more control
from galileo import galileo_context

with galileo_context(project="my-project", log_stream="my-log-stream"):
    response = openai.chat.completions.create(
        model="gpt-4o",
        messages=[
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": "Tell me about the solar system."}
        ]
    )
"""

import logging
import types
from collections import defaultdict
from dataclasses import dataclass
from datetime import datetime
from inspect import isclass
from typing import Any, Callable, Optional

from pydantic import BaseModel
from wrapt import wrap_function_wrapper  # type: ignore[import-untyped]

from galileo import GalileoLogger
from galileo.decorator import galileo_context
from galileo.utils import _get_timestamp
from galileo.utils.serialization import serialize_to_str

try:
    import openai
    import openai.resources
    from openai._types import NotGiven
    from openai.types.chat import ChatCompletionMessageToolCall

    # it's used only for version check of OpenAI
    from packaging.version import Version
except ImportError:
    raise ModuleNotFoundError("Please install OpenAI to use this feature: 'pip install openai'")

try:
    from openai import AsyncAzureOpenAI, AsyncOpenAI, AzureOpenAI, NoneType, OpenAI  # noqa: F401
except ImportError:
    AsyncAzureOpenAI = None  # type: ignore[assignment]
    AsyncOpenAI = None  # type: ignore[assignment]
    AzureOpenAI = None  # type: ignore[assignment]
    OpenAI = None  # type: ignore[assignment]


@dataclass
class OpenAiModuleDefinition:
    module: str
    object: str
    method: str
    type: str
    sync: bool
    min_version: Optional[str] = None


@dataclass
class OpenAiInputData:
    name: str
    metadata: dict
    start_time: datetime
    input: str
    model_parameters: dict
    model: Optional[str]
    temperature: float
    tools: Optional[list[dict]]


_logger = logging.getLogger(__name__)


OPENAI_CLIENT_METHODS = [
    OpenAiModuleDefinition(
        module="openai.resources.chat.completions", object="Completions", method="create", type="chat", sync=True
    )
    # Eventually add more OpenAI client library methods here
]


class OpenAiArgsExtractor:
    def __init__(self, name: Optional[str] = None, metadata: Optional[dict] = None, **kwargs: Any) -> None:
        self.args = {
            "name": name,
            "metadata": (
                metadata
                if "response_format" not in kwargs
                else {
                    **(metadata or {}),
                    "response_format": (
                        kwargs["response_format"].model_json_schema()
                        if isclass(kwargs["response_format"]) and issubclass(kwargs["response_format"], BaseModel)
                        else kwargs["response_format"]
                    ),
                }
            ),
        }
        self.kwargs = kwargs

    def get_galileo_args(self) -> dict[str, Any]:
        return {**self.args, **self.kwargs}

    def get_openai_args(self) -> dict[str, Any]:
        # If OpenAI model distillation is enabled, we need to add the metadata to the kwargs
        # https://platform.openai.com/docs/guides/distillation
        if self.kwargs.get("store", False):
            self.kwargs["metadata"] = self.args.get("metadata", {})

            # OpenAI does not support non-string type values in metadata when using
            # model distillation feature
            self.kwargs["metadata"].pop("response_format", None)

        return self.kwargs


def _galileo_wrapper(func: Callable) -> Callable:
    def _with_galileo(open_ai_definitions: OpenAiModuleDefinition, initialize: Callable) -> Callable:
        def wrapper(wrapped: Callable, instance: Any, args: dict, kwargs: dict) -> Any:
            return func(open_ai_definitions, initialize, wrapped, args, kwargs)

        return wrapper

    return _with_galileo


def _extract_chat_response(kwargs: dict) -> dict:
    """Extracts the llm output from the response."""
    response = {"role": kwargs.get("role", None)}

    if kwargs.get("function_call") is not None and type(kwargs["function_call"]) is dict:
        response.update(
            {
                "tool_calls": [
                    {
                        "id": "",
                        "function": {
                            "name": kwargs["function_call"].get("name", ""),
                            "arguments": kwargs["function_call"].get("arguments", ""),
                        },
                    }
                ]
            }
        )
    elif kwargs.get("tool_calls") is not None and type(kwargs["tool_calls"]) is list:
        tool_calls = []
        for tool_call in kwargs["tool_calls"]:
            try:
                tool_call = ChatCompletionMessageToolCall.model_validate(tool_call)
                tool_calls.append(
                    {
                        "id": tool_call.id,
                        "function": {"name": tool_call.function.name, "arguments": tool_call.function.arguments},
                    }
                )
            except Exception as e:
                _logger.error(f"Error processing tool call: {e}")

        response.update({"tool_calls": tool_calls if len(tool_calls) else None})

    response.update({"content": kwargs.get("content", "")})

    return response


def _extract_input_data_from_kwargs(
    resource: OpenAiModuleDefinition, start_time: datetime, kwargs: dict[str, Any]
) -> OpenAiInputData:
    name: str = kwargs.get("name", "openai-client-generation")

    if name is not None and not isinstance(name, str):
        raise TypeError("name must be a string")

    metadata: dict = kwargs.get("metadata", {})

    if metadata is not None and not isinstance(metadata, dict):
        raise TypeError("metadata must be a dictionary")

    model = kwargs.get("model", None) or None

    prompt = None

    if resource.type == "completion":
        prompt = kwargs.get("prompt", None)
    elif resource.type == "chat":
        prompt = kwargs.get("messages", [])

    parsed_temperature = float(
        kwargs.get("temperature", 1) if not isinstance(kwargs.get("temperature", 1), NotGiven) else 1
    )

    parsed_max_tokens = (
        kwargs.get("max_tokens", float("inf"))
        if not isinstance(kwargs.get("max_tokens", float("inf")), NotGiven)
        else float("inf")
    )

    parsed_top_p = kwargs.get("top_p", 1) if not isinstance(kwargs.get("top_p", 1), NotGiven) else 1

    parsed_frequency_penalty = (
        kwargs.get("frequency_penalty", 0) if not isinstance(kwargs.get("frequency_penalty", 0), NotGiven) else 0
    )

    parsed_presence_penalty = (
        kwargs.get("presence_penalty", 0) if not isinstance(kwargs.get("presence_penalty", 0), NotGiven) else 0
    )

    parsed_seed = kwargs.get("seed", None) if not isinstance(kwargs.get("seed", None), NotGiven) else None

    parsed_n = kwargs.get("n", 1) if not isinstance(kwargs.get("n", 1), NotGiven) else 1

    parsed_tools = kwargs.get("tools", None) if not isinstance(kwargs.get("tools", None), NotGiven) else None

    parsed_tool_choice = (
        kwargs.get("tool_choice", None) if not isinstance(kwargs.get("tool_choice", None), NotGiven) else None
    )

    # handle deprecated aliases (functions for tools, function_call for tool_choice)
    if parsed_tools is None and kwargs.get("functions") is not None:
        parsed_tools = kwargs["functions"]

    if parsed_tool_choice is None and kwargs.get("function_call") is not None:
        parsed_tool_choice = kwargs["function_call"]

    model_parameters = {
        "temperature": parsed_temperature,
        "max_tokens": parsed_max_tokens,
        "top_p": parsed_top_p,
        "frequency_penalty": parsed_frequency_penalty,
        "presence_penalty": parsed_presence_penalty,
        "tool_choice": parsed_tool_choice,
    }
    if parsed_n is not None and parsed_n > 1:
        model_parameters["n"] = parsed_n

    if parsed_seed is not None:
        model_parameters["seed"] = parsed_seed

    return OpenAiInputData(
        name=name,
        metadata=metadata,
        start_time=start_time,
        # TODO: galileo/openai.py:229: error:
        # Argument "input" to "OpenAiInputData" has incompatible type "Any | None"; expected "str"  [arg-type]
        input=prompt,  # type: ignore[arg-type]
        model_parameters=model_parameters,
        model=model or None,
        temperature=parsed_temperature,
        tools=parsed_tools,
    )


def _parse_usage(usage: Optional[dict] = None) -> Optional[dict]:
    if usage is None:
        return None

    usage_dict = usage.copy() if isinstance(usage, dict) else usage.__dict__

    for tokens_details in ["prompt_tokens_details", "completion_tokens_details"]:
        if tokens_details in usage_dict and usage_dict[tokens_details] is not None:
            tokens_details_dict = (
                usage_dict[tokens_details]
                if isinstance(usage_dict[tokens_details], dict)
                else usage_dict[tokens_details].__dict__
            )
            usage_dict[tokens_details] = {k: v for k, v in tokens_details_dict.items() if v is not None}

    return usage_dict


def _extract_data_from_default_response(resource: OpenAiModuleDefinition, response: dict[str, Any]) -> Any:
    if response is None:
        return None, "<NoneType response returned from OpenAI>", None

    model = response.get("model", None) or None

    completion = None
    if resource.type == "completion":
        choices = response.get("choices", [])
        if len(choices) > 0:
            choice = choices[-1]

            completion = choice.text if _is_openai_v1() else choice.get("text", None)
    elif resource.type == "chat":
        choices = response.get("choices", [])
        if len(choices):
            if len(choices) > 1:
                completion = [
                    (
                        _extract_chat_response(choice.message.__dict__)
                        if _is_openai_v1()
                        else choice.get("message", None)
                    )
                    for choice in choices
                ]
            else:
                choice = choices[0]
                completion = (
                    _extract_chat_response(choice.message.__dict__) if _is_openai_v1() else choice.get("message", None)
                )

    usage = _parse_usage(response.get("usage", None))

    return model, completion, usage


def _extract_streamed_openai_response(resource, chunks):
    completion = defaultdict(str) if resource.type == "chat" else ""
    model, usage = None, None

    for chunk in chunks:
        if _is_openai_v1():
            chunk = chunk.__dict__

        model = model or chunk.get("model", None) or None
        usage = chunk.get("usage", None)

        choices = chunk.get("choices", [])

        for choice in choices:
            if _is_openai_v1():
                choice = choice.__dict__
            if resource.type == "chat":
                delta = choice.get("delta", None)

                if _is_openai_v1():
                    delta = delta.__dict__

                if delta.get("role", None) is not None:
                    completion["role"] = delta["role"]

                if delta.get("content", None) is not None:
                    completion["content"] = (
                        delta.get("content", None)
                        if completion["content"] is None
                        else completion["content"] + delta.get("content", None)
                    )
                elif delta.get("function_call", None) is not None:
                    curr = completion["function_call"]
                    tool_call_chunk = delta.get("function_call", None)

                    if not curr:
                        completion["function_call"] = {
                            "name": getattr(tool_call_chunk, "name", ""),
                            "arguments": getattr(tool_call_chunk, "arguments", ""),
                        }

                    else:
                        curr["name"] = curr["name"] or getattr(tool_call_chunk, "name", None)
                        curr["arguments"] += getattr(tool_call_chunk, "arguments", "")

                elif delta.get("tool_calls", None) is not None:
                    curr = completion["tool_calls"]
                    tool_call_chunk = getattr(delta.get("tool_calls", None)[0], "function", None)

                    if not curr:
                        completion["tool_calls"] = [
                            {
                                "name": getattr(tool_call_chunk, "name", ""),
                                "arguments": getattr(tool_call_chunk, "arguments", ""),
                            }
                        ]

                    elif getattr(tool_call_chunk, "name", None) is not None:
                        curr.append(
                            {
                                "name": getattr(tool_call_chunk, "name", None),
                                "arguments": getattr(tool_call_chunk, "arguments", None),
                            }
                        )

                    else:
                        curr[-1]["name"] = curr[-1]["name"] or getattr(tool_call_chunk, "name", None)
                        curr[-1]["arguments"] += getattr(tool_call_chunk, "arguments", None)

            if resource.type == "completion":
                completion += choice.get("text", None)

    def get_response_for_chat():
        return (
            completion["content"]
            or (completion["function_call"] and {"role": "assistant", "function_call": completion["function_call"]})
            or (
                completion["tool_calls"]
                and {"role": "assistant", "tool_calls": [{"function": data} for data in completion["tool_calls"]]}
            )
            or None
        )

    return (model, get_response_for_chat() if resource.type == "chat" else completion, usage)


def _is_openai_v1() -> bool:
    return Version(openai.__version__) >= Version("1.0.0")


def _is_streaming_response(response):
    return isinstance(response, types.GeneratorType) or (_is_openai_v1() and isinstance(response, openai.Stream))


@_galileo_wrapper
def _wrap(
    open_ai_resource: OpenAiModuleDefinition, initialize: Callable, wrapped: Callable, args: dict, kwargs: dict
) -> Any:
    # Retrieve the decorator context
    decorator_context_trace = galileo_context.get_current_trace()

    start_time = _get_timestamp()
    arg_extractor = OpenAiArgsExtractor(*args, **kwargs)

    input_data = _extract_input_data_from_kwargs(open_ai_resource, start_time, arg_extractor.get_galileo_args())

    galileo_logger: GalileoLogger = initialize()

    should_complete_trace = False
    if decorator_context_trace:
        pass
    else:
        # If we don't have an active trace, start a new trace
        # We will conclude it at the end
        galileo_logger.start_trace(input=serialize_to_str(input_data.input), name=input_data.name)
        should_complete_trace = True

    try:
        openai_response = wrapped(**arg_extractor.get_openai_args())

        if _is_streaming_response(openai_response):
            # extract data from streaming reponse
            return ResponseGeneratorSync(
                resource=open_ai_resource,
                response=openai_response,
                input_data=input_data,
                logger=galileo_logger,
                should_complete_trace=should_complete_trace,
            )
        else:
            model, completion, usage = _extract_data_from_default_response(
                open_ai_resource,
                ((openai_response and openai_response.__dict__) if _is_openai_v1() else openai_response),
            )

            if usage is None:
                usage = {}

            end_time = _get_timestamp()

            duration_ns = int(round((end_time - start_time).total_seconds() * 1e9))

            # Add a span to the current trace or span (if this is a nested trace)
            galileo_logger.add_llm_span(
                input=input_data.input,
                output=completion,
                tools=input_data.tools,
                name=input_data.name,
                model=model,
                temperature=input_data.temperature,
                duration_ns=duration_ns,
                num_input_tokens=usage.get("prompt_tokens", 0),
                num_output_tokens=usage.get("completion_tokens", 0),
                total_tokens=usage.get("total_tokens", 0),
                metadata={str(k): str(v) for k, v in input_data.model_parameters.items()},
                # openai client library doesn't return http_status code, so we only can hardcode it here
                # because we if we parsed and extracted data from response it means we get it and it's 200OK
                status_code=200,
            )

            # Conclude the trace if this is the top-level call
            if should_complete_trace:
                galileo_logger.conclude(output=serialize_to_str(completion), duration_ns=duration_ns)

        return openai_response
    except Exception as ex:
        _logger.error(f"Error while processing OpenAI request: {ex}")
        raise RuntimeError("Failed to process the OpenAI Request") from ex


class ResponseGeneratorSync:
    """
    A wrapper for OpenAI streaming responses that logs the response to Galileo.

    This class wraps the OpenAI streaming response generator and logs the response
    to Galileo when the generator is exhausted. It implements the iterator protocol
    to allow for streaming responses.

    Attributes
    ----------
    resource : OpenAiModuleDefinition
        The OpenAI resource definition.
    response : Generator or openai.Stream
        The OpenAI streaming response.
    input_data : OpenAiInputData
        The input data for the OpenAI request.
    logger : GalileoLogger
        The Galileo logger instance.
    should_complete_trace : bool
        Whether to complete the trace when the generator is exhausted.
    """

    def __init__(self, *, resource, response, input_data, logger: GalileoLogger, should_complete_trace: bool):
        self.items = []
        self.resource = resource
        self.response = response
        self.input_data = input_data
        self.logger = logger
        self.should_complete_trace = should_complete_trace
        self.completion_start_time = None

    def __iter__(self):
        try:
            for i in self.response:
                self.items.append(i)

                if self.completion_start_time is None:
                    self.completion_start_time = _get_timestamp()

                yield i
        finally:
            self._finalize()

    def __next__(self):
        try:
            item = self.response.__next__()
            self.items.append(item)

            if self.completion_start_time is None:
                self.completion_start_time = _get_timestamp()

            return item

        except StopIteration:
            self._finalize()

            raise

    def __enter__(self):
        return self.__iter__()

    def __exit__(self, exc_type, exc_value, traceback):
        pass

    def _finalize(self):
        model, completion, usage = _extract_streamed_openai_response(self.resource, self.items)

        if usage is None:
            usage = {}

        end_time = _get_timestamp()
        # TODO: make sure completion_start_time what we want
        duration_ns = int(round((end_time - self.completion_start_time).total_seconds() * 1e9))

        # Add a span to the current trace or span (if this is a nested trace)
        self.logger.add_llm_span(
            input=self.input_data.input,
            output=completion,
            tools=self.input_data.tools,
            name=self.input_data.name,
            model=model,
            temperature=self.input_data.temperature,
            duration_ns=duration_ns,
            num_input_tokens=usage.get("prompt_tokens", 0),
            num_output_tokens=usage.get("completion_tokens", 0),
            total_tokens=usage.get("total_tokens", 0),
            metadata={str(k): str(v) for k, v in self.input_data.model_parameters.items()},
            status_code=200,
        )

        # Conclude the trace if this is the top-level call
        if self.should_complete_trace:
            self.logger.conclude(output=completion, duration_ns=duration_ns)


class OpenAIGalileo:
    """
    This class is responsible for logging OpenAI API calls and logging them to Galileo.
    It wraps the OpenAI client methods to add logging functionality without changing
    the original API behavior.

    Attributes
    ----------
    _galileo_logger : Optional[GalileoLogger]
        The Galileo logger instance used for logging OpenAI API calls.
    """

    _galileo_logger: Optional[GalileoLogger] = None

    def initialize(self) -> Optional[GalileoLogger]:
        """
        Initialize a Galileo logger.

        Parameters
        ----------
        project : Optional[str]
            The project to log to. If None, uses the default project.
        log_stream : Optional[str]
            The log stream to log to. If None, uses the default log stream.

        Returns
        -------
        Optional[GalileoLogger]
            The initialized Galileo logger instance.
        """
        self._galileo_logger = galileo_context.get_logger_instance()

        return self._galileo_logger

    def register_tracing(self) -> None:
        """
        This method wraps the OpenAI client methods to intercept calls and log them to Galileo.
        It is called automatically when the module is imported.

        The wrapped methods include:
        - openai.resources.chat.completions.Completions.create

        Additional methods can be added to the OPENAI_CLIENT_METHODS list.
        """
        for resource in OPENAI_CLIENT_METHODS:
            wrap_function_wrapper(
                resource.module, f"{resource.object}.{resource.method}", (_wrap(resource, self.initialize))
            )


modifier = OpenAIGalileo()
modifier.register_tracing()



from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="LassoSelection")


@_attrs_define
class LassoSelection:
    """Representation of a lasso selection (used by embeddings)

    x and y correspond to the cursor movement while tracing the lasso. This is natively provided by plotly when creating
    a lasso selection, for example

        Attributes:
            x (list[float]):
            y (list[float]):
    """

    x: list[float]
    y: list[float]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        x = self.x

        y = self.y

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"x": x, "y": y})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        x = cast(list[float], d.pop("x"))

        y = cast(list[float], d.pop("y"))

        lasso_selection = cls(x=x, y=y)

        lasso_selection.additional_properties = d
        return lasso_selection

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class FieldType(str, Enum):
    ARRAY = "array"
    BOOL = "bool"
    FLOAT = "float"
    INT = "int"
    STRING = "string"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.get_token_response import GetTokenResponse
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Response, Unset


def _get_kwargs(*, organization_id: Union[None, Unset, str] = UNSET) -> dict[str, Any]:
    params: dict[str, Any] = {}

    json_organization_id: Union[None, Unset, str]
    if isinstance(organization_id, Unset):
        json_organization_id = UNSET
    else:
        json_organization_id = organization_id
    params["organization_id"] = json_organization_id

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {"method": "get", "url": "/token", "params": params}

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[GetTokenResponse, HTTPValidationError]]:
    if response.status_code == 200:
        response_200 = GetTokenResponse.from_dict(response.json())

        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[GetTokenResponse, HTTPValidationError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *, client: AuthenticatedClient, organization_id: Union[None, Unset, str] = UNSET
) -> Response[Union[GetTokenResponse, HTTPValidationError]]:
    """Get Token

    Args:
        organization_id (Union[None, Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[GetTokenResponse, HTTPValidationError]]
    """

    kwargs = _get_kwargs(organization_id=organization_id)

    response = client.get_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


def sync(
    *, client: AuthenticatedClient, organization_id: Union[None, Unset, str] = UNSET
) -> Optional[Union[GetTokenResponse, HTTPValidationError]]:
    """Get Token

    Args:
        organization_id (Union[None, Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[GetTokenResponse, HTTPValidationError]
    """

    return sync_detailed(client=client, organization_id=organization_id).parsed


async def asyncio_detailed(
    *, client: AuthenticatedClient, organization_id: Union[None, Unset, str] = UNSET
) -> Response[Union[GetTokenResponse, HTTPValidationError]]:
    """Get Token

    Args:
        organization_id (Union[None, Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[GetTokenResponse, HTTPValidationError]]
    """

    kwargs = _get_kwargs(organization_id=organization_id)

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *, client: AuthenticatedClient, organization_id: Union[None, Unset, str] = UNSET
) -> Optional[Union[GetTokenResponse, HTTPValidationError]]:
    """Get Token

    Args:
        organization_id (Union[None, Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[GetTokenResponse, HTTPValidationError]
    """

    return (await asyncio_detailed(client=client, organization_id=organization_id)).parsed
